#ifndef _USBI_H_
#define _USBI_H_

#include "usb.h"

#include "error.h"

extern int usb_debug;

/* Some quick and generic macros for the simple kind of lists we use */
#define LIST_ADD(begin, ent) \
	do { \
	  if (begin) { \
	    ent->next = begin; \
	    ent->next->prev = ent; \
	  } else \
	    ent->next = NULL; \
	  ent->prev = NULL; \
	  begin = ent; \
	} while(0)

#define LIST_DEL(begin, ent) \
	do { \
	  if (ent->prev) { \
	    ent->prev->next = ent->next; \
	    ent->prev = NULL; \
	  } else \
	    begin = ent->next; \
	  if (ent->next) { \
	    ent->next->prev = ent->prev; \
	    ent->next = NULL; \
	  } \
	} while (0)

/* All of these structures are used for internal purposes and MUST not be */
/* leaked out to userspace so we can maintain forward compatibility */
struct usbi_device;

struct usbi_bus {
  struct usbi_bus *prev, *next;

  char filename[PATH_MAX + 1];
  void *priv;

  struct usbi_device *devices;
};

struct usbi_device {
  struct usbi_device *prev, *next;

  struct usb_device *dev;

  char filename[PATH_MAX + 1];
  void *priv;

  struct usbi_bus *bus;

  struct usb_device_descriptor descriptor;
  struct usb_config_descriptor *config;
};

struct usbi_dev_handle {
  int fd;

  struct usbi_bus *bus;
  struct usbi_device *device;

  void *priv;

  /* These are the currently configured values */
  int config;
  int interface;
  int altsetting;
};

struct usb_match {
  int idVendor;
  int idProduct;

  int bClass;
  int bSubClass;
  int bProtocol;
};

void usb_fetch_descriptors(usb_dev_handle *dev);

/* OS specific routines */
int usb_os_find_busses(struct usbi_bus **busses);
int usb_os_find_devices(struct usbi_bus *bus, struct usbi_device **devices);
void usb_os_init(void);
int usb_os_open(usb_dev_handle *dev);
int usb_os_close(usb_dev_handle *dev);

void usb_free_dev(struct usbi_device *dev);
void usb_free_bus(struct usbi_bus *bus);

#endif /* _USBI_H_ */

